@@ -1,31 +1,33 @@
 /*************************************************************************
-***     Authentication, authorization, accounting + firewalling package
-***     (c) 1998-2003 Anton Vinokurov <anton@netams.com>
-***		(c) 2003 NeTAMS Development Team
-***		All rights reserved. See 'Copying' file included in distribution
-***		For latest version and more info, visit this project web page
-***		located at http://www.netams.com
-***		 					
+***	Authentication, authorization, accounting + firewalling package
+***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
+***	(c) 2002-2005 NeTAMS Development Team
+***	All rights reserved. See 'Copying' file included in distribution
+***	For latest version and more info, visit this project web page
+***	located at http://www.netams.com
+***
 *************************************************************************/
-/* $Id: ds_mailstat.c,v 1.2.2.1 2004/05/06 10:35:02 jura Exp $ */
+/* $Id: ds_mailstat.c,v 1.5 2005/01/17 13:13:21 jura Exp $ */
 
 #include "netams.h"
 
+/////////////////////////////////////////////////////////////////////////////////////
+void ds_mail_cancel(void *ptr);
+void dsMail(Service *s, struct ip *ip);
+/////////////////////////////////////////////////////////////////////////////////////
+void* ds_mail(void *ss) {
+//	Service *s=(Service*)ss;
+//	ServiceDS_cfg *cfg=(ServiceDS_cfg*)s->cfg;
+
+//	pthread_cleanup_push(ds_mail_cancel, (void*)NULL);
+	return NULL;
+}
+/////////////////////////////////////////////////////////////////////////////////////
+void ds_mail_cancel(void *ptr) {
+}
 /////////////////////////////////////////////////////////////////////////////////////
 void dsMail(Service *s, struct ip *ip){
-
-	}										
-
+}										
 /////////////////////////////////////////////////////////////////////////////////////
-void dsMailHookAdd(Service *s){
-
-	}
-
-/////////////////////////////////////////////////////////////////////////////////////
-void dsMailHookRemove(Service *s){
-
-	}
-
 /////////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////////////
-
