@@ -1,17 +1,21 @@
 /*************************************************************************
-***     Authentication, authorization, accounting + firewalling package
-***     (c) 1998-2003 Anton Vinokurov <anton@netams.com>
-***		(c) 2003 NeTAMS Development Team
-***		All rights reserved. See 'Copying' file included in distribution
-***		For latest version and more info, visit this project web page
-***		located at http://www.netams.com
-***		 					
+***	Authentication, authorization, accounting + firewalling package
+***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
+***	(c) 2002-2005 NeTAMS Development Team
+***	All rights reserved. See 'Copying' file included in distribution
+***	For latest version and more info, visit this project web page
+***	located at http://www.netams.com
+***
 *************************************************************************/
-/* $Id: policy_prefix.h,v 1.8.2.2 2004/07/12 07:54:26 jura Exp $ */
+/* $Id: policy_prefix.h,v 1.14 2005/01/17 13:13:21 jura Exp $ */
+
+#define PREFIX_COMMENTED	0x01
+#define PREFIX_INVERTED		0x02
 
 typedef struct prefix {
-	in_addr_t network;
+	in_addr network;
 	u_char mask;
+	u_char flags;
 	unsigned long long match;
 } prefix;
 
