@@ -1,18 +1,22 @@
 /*************************************************************************
-***     Authentication, authorization, accounting + firewalling package
-***     (c) 1998-2003 Anton Vinokurov <anton@netams.com>
-***		(c) 2003 NeTAMS Development Team
-***		All rights reserved. See 'Copying' file included in distribution
-***		For latest version and more info, visit this project web page
-***		located at http://www.netams.com
-***		 					
+***	Authentication, authorization, accounting + firewalling package
+***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
+***	(c) 2002-2005 NeTAMS Development Team
+***	All rights reserved. See 'Copying' file included in distribution
+***	For latest version and more info, visit this project web page
+***	located at http://www.netams.com
+***
 *************************************************************************/
-/* $Id: s_quotactl.c,v 1.20.2.6 2004/06/21 09:52:36 jura Exp $ */
+/* $Id: s_quotactl.c,v 1.52 2005/01/17 13:13:22 jura Exp $ */
 
 #include "netams.h"
 
 Quotactl_cfg QuotactlCfg;
-
+//////////////////////////////////////////////////////////////////////////
+void *sQuotactl(void *s);
+void sQuotactlCancel(void *);
+void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, u_char dir);
+void sQuotactlGetValue(u_char *i, char *param[], qstat *q);
 //////////////////////////////////////////////////////////////////////////
 void sQuotactlInit(Service *s){
 	s->cfg = (Quotactl_cfg*)aMalloc(sizeof(Quotactl_cfg));
@@ -33,42 +37,43 @@ void sQuotactlInit(Service *s){
 	}
 
 //////////////////////////////////////////////////////////////////////////
-void sQuotactlProcessCfg(char *param[32], Connection *conn, unsigned no_flag){
+void sQuotactlProcessCfg(char *param[], Connection *conn, u_char no_flag){
 	Quotactl_cfg *cfg=(Quotactl_cfg*)conn->service->cfg;
 	Qdef *q=NULL;
 	Qdef *qq=NULL;
+	time_t now=time(NULL);
 
 	pthread_rwlock_wrlock(cfg->rwlock);
 	
 	if (!strcasecmp(param[0], "quota") && param[1]!=empty) {
-		for (Qdef *quotas=cfg->root; quotas!=NULL; quotas=quotas->next) {
-			qq=quotas;
-			if (quotas->name && strcasecmp(param[1], quotas->name)==0) q=quotas;
+		for (q=cfg->root; q!=NULL; q=q->next) {
+			if (q->name && strcasecmp(param[1], q->name)==0) break;
+			qq=q;
 		}
 
-		if (q && no_flag) { 
+		if (q && no_flag) {
 			if(q==cfg->root) cfg->root=q->next;
-			else q->prev->next=q->next;
-			if(q->next) q->next->prev=q->prev;
-			for(qUlist *ql=q->root;ql!=NULL;ql=ql->next) {
+			else qq->next=q->next;
+			qUlist *ql=q->root;
+			qUlist *tmp;
+			while(ql){
 				NetUnit *u=ql->u;
-		        	if (u && u->sys_policy==q->sys_policy) {
-                                	u->sys_policy=SP_NONE;
-                                        u->sys_policy_name=SysPolicy_name[SP_NONE];
-                                        u->sys_policy_perm=0;
+		        	if (u && (u->sys_policy&SP_DENY_QUOTA)) {
+					u->SetSysPolicy(SP_DENY_QUOTA, REMOVE, now);
+					if(q->sys_policy_perm) u->sys_policy_perm=0;
 					aParse(conn, "unit %06X (%s) unbinded from quota %s\n", u->id, u->name?u->name:"<>", q->name);
                                 }
-				aFree(ql);
+				tmp=ql;
+				ql=ql->next;
+				aFree(tmp);
 			}
 			aParse(conn, "quota %s deleted\n", q->name);
 			aFree(q->name);
 			aFree(q); 
 			pthread_rwlock_unlock(cfg->rwlock);
 			return;
-		}	
-
+		}
 		if (!q) {
-
 			if (no_flag) { 
 				aParse(conn, "quota %s not defined\n", param[1]);
 				pthread_rwlock_unlock(cfg->rwlock);
@@ -76,16 +81,14 @@ void sQuotactlProcessCfg(char *param[32], Connection *conn, unsigned no_flag){
 			}	
 
 			q=(Qdef*) aMalloc(sizeof(Qdef));
-			bzero(q,sizeof(Qdef)); //clear stats    
 			q->name=set_string(param[1]);
 			if (!cfg->root) cfg->root=q; else qq->next=q;
-			q->prev=qq;
 			q->next=NULL;
 			q->root=NULL;
 			aParse(conn, "quota %s created\n", q->name);
 		}
 
-		int i=2;
+		u_char i=2;
 		while (param[i]!=empty) {
 
 	 		if (strcasecmp(param[i], "policy")==0) {
@@ -100,20 +103,16 @@ void sQuotactlProcessCfg(char *param[32], Connection *conn, unsigned no_flag){
 				NetUnit *u = new NetUnit(NETUNIT_UNKNOWN);
 				u->id=0xFFFFFF;
 				SetSysPolicy(param[i+1], u, conn);
-				q->sys_policy=u->sys_policy;
-				q->sys_policy_name=u->sys_policy_name;
+				q->sys_policy|=u->sys_policy;
 				q->sys_policy_perm=u->sys_policy_perm;
-				
-				if (q->sys_policy_perm) aParse(conn, "quota %s set: sys-LOCAL%s%s, allowed OID %06X\n", q->name, q->sys_policy_name[0]?"-":"", q->sys_policy_name, q->sys_policy_perm);
-				else aParse(conn, "quota %s set: %s\n", q->name, q->sys_policy_name);
-
+				aParse(conn, "quota %s set: ", q->name);
+				PrintSysPolicy(conn->stream_w, u->sys_policy, u->sys_policy_perm);
 				delete u;
 			}
 			else if (strcasecmp(param[i], "hour")==0) sQuotactlGetValue(&i, param, &(q->h));
 			else if (strcasecmp(param[i], "day")==0) sQuotactlGetValue(&i, param, &(q->d));
 			else if (strcasecmp(param[i], "week")==0) sQuotactlGetValue(&i, param, &(q->w));
 			else if (strcasecmp(param[i], "month")==0) sQuotactlGetValue(&i, param, &(q->m));
-			else if (strcasecmp(param[i], "total")==0) sQuotactlGetValue(&i, param, &(q->t));
 
 			i+=2;
 		} // while
@@ -121,54 +120,52 @@ void sQuotactlProcessCfg(char *param[32], Connection *conn, unsigned no_flag){
 	else if (!strcasecmp(param[0], "bind") && !strcasecmp(param[1], "quota") && !strcasecmp(param[3], "to")) {
 		for (Qdef *quotas=cfg->root; quotas!=NULL; quotas=quotas->next) {
 			if (quotas->name && strcasecmp(param[2], quotas->name)==0) q=quotas;
-		}
+			}
 		if (!q) { aParse(conn, "bind to undefined quota\n"); pthread_rwlock_unlock(cfg->rwlock); return; }
-		int i=4;
+		u_short i=4;
 		NetUnit *u; oid id;
 		while (param[i]!=empty) {
-			qUlist *ql,*qu,*qql;
+			qUlist *ql,*qu,*qql=NULL;
 			id=strtol(param[i], NULL, 16);
 			u=Units.getUnitById(id);
 			if (!u) u=Units.getUnit(param[i]);
 			if (u) {
 				for (ql=q->root; ql!=NULL; ql=ql->next) {
-					if (!no_flag && ql->u==u) {
+					if (ql->u==u) break;
+					qql=ql;
+				}
+				if(ql) { 
+					if(!no_flag) {
 						aParse(conn, "unit %06X (%s) is already binded with quota %s\n", u->id, u->name?u->name:"<>", q->name);
 						goto AGAIN;
 					}
-				}
-				for (ql=q->root; ql!=NULL; ql=ql->next) {
-					if (no_flag) {
-						if (ql->u==u) {
-							if(q->root == ql) q->root=ql->next; else ql->prev->next=ql->next;
-							if(ql->next) ql->next->prev=ql->prev;
-                                			if (u && u->sys_policy==q->sys_policy) {
-                                        			u->sys_policy=SP_NONE;
-                                        			u->sys_policy_name=SysPolicy_name[SP_NONE];
-                                        			u->sys_policy_perm=0;
-                                			}
-							aFree(ql);
-							aParse(conn, "unit %06X (%s) unbinded from quota %s\n", u->id, u->name?u->name:"<>", q->name);
-							goto AGAIN;
-							}
-						}
-					qql=ql;
+				
+					if(q->root == ql) q->root=ql->next; 
+					else qql->next=ql->next;
+                                	
+					if (u && (u->sys_policy&SP_DENY_QUOTA)) {
+						u->SetSysPolicy(SP_DENY_QUOTA, REMOVE, now);
+						if(q->sys_policy_perm) u->sys_policy_perm=0;
+                                	}
+					aFree(ql);
+					aParse(conn, "unit %06X (%s) unbinded from quota %s\n", u->id, u->name?u->name:"<>", q->name);
+					goto AGAIN;
 				}
 				qu=(qUlist*)aMalloc(sizeof(qUlist));
 				qu->u=u;
-				qu->next=NULL;
-				if(!q->root) q->root=qu; else qql->next=qu;
-				qu->prev=qql;
+				qu->next=q->root; 
+				q->root=qu;
 				qu->q_soft=qu->q_viol=0;
 				aParse(conn, "unit %06X (%s) binded with quota %s\n", u->id, u->name?u->name:"<>", q->name);
 			}
 			else aParse(conn, "unit %06X unknown, cannot bind\n", id);
-AGAIN:		i++;
+AGAIN:			
+			i++;
 		}		
 	}
 	else if (!strcasecmp(param[0], "alert")) {
 		User *x;
-		int j;
+		u_short j;
 
 		if (!strcasecmp(param[1], "soft-reach")) {
 			if (!strcasecmp(param[2], "<user>")) { j=3;	cfg->user_soft_reach=1; }
@@ -178,7 +175,8 @@ AGAIN:		i++;
 			if (!x) x=Users.getUserById(strtol(param[j], NULL, 16));
 			if (!x && param[j]!=empty) aParse(conn, "user %s unknown\n", param[j]);
 			else if (x) cfg->soft_reach=x;
-		}
+			}
+
 		else if (!strcasecmp(param[1], "hard-reach")) {
 			if (!strcasecmp(param[2], "<user>")) { j=3;	cfg->user_hard_reach=1; }
 			else j=2; 
@@ -187,7 +185,7 @@ AGAIN:		i++;
 			if (!x) x=Users.getUserById(strtol(param[j], NULL, 16));
 			if (!x && param[j]!=empty) aParse(conn, "user %s unknown\n", param[j]);
 			else if (x) cfg->hard_reach=x;
-		}
+			}
 		else if (!strcasecmp(param[1], "reset-back")) {
 			if (!strcasecmp(param[2], "<user>")) { j=3;	cfg->user_reset_back=1; }
 			else j=2; 
@@ -196,22 +194,25 @@ AGAIN:		i++;
 			if (!x) x=Users.getUserById(strtol(param[j], NULL, 16));
 			if (!x && param[j]!=empty) aParse(conn, "user %s unknown\n", param[j]);
 			else if (x) cfg->reset_back=x;
-		}
+			}
 		else aParse(conn, "quotactl alert parameter unknown: %s\n", param[1]);
-	}
+		}
+
 	else if (!strcasecmp(param[0], "lookup-delay")) {
-		unsigned delay=atoi(param[1]);
+		unsigned delay=strtol(param[1], NULL, 10);
 		if(delay>1 && delay <24*60*60) {
-			cfg->delay=delay*1000;
+			cfg->delay=delay;
 			aParse(conn, "quotas will be checked every %u seconds\n",delay);
-		} 
+			} 
 		else aParse(conn, "lookup delay value invalid\n");
-	}
+		}
+
 	else aParse(conn, "unknown quotactl command: %s\n", param[0]);
 	
 	pthread_rwlock_unlock(cfg->rwlock);
 	return;
-}
+	}
+
 //////////////////////////////////////////////////////////////////////////
 void sQuotactlListCfg(Service *s, FILE *f){
 	Quotactl_cfg *cfg=(Quotactl_cfg*)s->cfg;
@@ -221,16 +222,14 @@ void sQuotactlListCfg(Service *s, FILE *f){
 	if (!cfg->rwlock) return;
 
 	pthread_rwlock_rdlock(cfg->rwlock);
-	fprintf(f, "service quotactl %d\n", s->instance);
-	if(cfg->delay!=QUOTACTL_DELAY) fprintf(f, "lookup-delay %d\n", cfg->delay/1000);
+	if(cfg->delay!=QUOTACTL_DELAY) fprintf(f, "lookup-delay %d\n", cfg->delay);
 	for (q=cfg->root; q!=NULL; q=q->next) {
 		if (q->name) {
 			fprintf(f, "quota %s policy %s ", q->name, (q->policy && q->policy->name)?q->policy->name:"<>");
 
-			if (q->sys_policy!=SP_NONE) { 
-				if (q->sys_policy_perm) fprintf(f, "set sys-%06X%s%s ", q->sys_policy_perm, q->sys_policy_name[0]?"-":"", q->sys_policy_name);
-				else fprintf(f, "set %s ", q->sys_policy_name);
-				}
+			if (q->sys_policy) {
+				PrintSysPolicy(f, q->sys_policy, q->sys_policy_perm);
+			}
 
 			if (q->h.in || q->h.out || q->h.softin || q->h.softout || q->h.sum || q->h.softsum) {
 				fprintf(f, "hour ");
@@ -240,7 +239,7 @@ void sQuotactlListCfg(Service *s, FILE *f){
 				if (q->h.in) fprintf(f, "%s in ", bytesQ2T(q->h.in, buf));
 				if (q->h.out) fprintf(f, "%s out ", bytesQ2T(q->h.out, buf));
 				if (q->h.sum) fprintf(f, "%s sum ", bytesQ2T(q->h.sum, buf));
-				}
+			}
 		 
 			if (q->d.in || q->d.out || q->d.softin || q->d.softout || q->d.sum || q->d.softsum) {
 				fprintf(f, "day ");
@@ -250,7 +249,7 @@ void sQuotactlListCfg(Service *s, FILE *f){
 				if (q->d.in) fprintf(f, "%s in ", bytesQ2T(q->d.in, buf));
 				if (q->d.out) fprintf(f, "%s out ", bytesQ2T(q->d.out, buf));
 				if (q->d.sum) fprintf(f, "%s sum ", bytesQ2T(q->d.sum, buf));
-				}
+			}
 
 			if (q->w.in || q->w.out || q->w.softin || q->w.softout || q->w.sum || q->w.softsum) {
 				fprintf(f, "week ");
@@ -260,7 +259,7 @@ void sQuotactlListCfg(Service *s, FILE *f){
 				if (q->w.in) fprintf(f, "%s in ", bytesQ2T(q->w.in, buf));
 				if (q->w.out) fprintf(f, "%s out ", bytesQ2T(q->w.out, buf));
 				if (q->w.sum) fprintf(f, "%s sum ", bytesQ2T(q->w.sum, buf));
-				}
+			}
 
 			if (q->m.in || q->m.out || q->m.softin || q->m.softout || q->m.sum || q->m.softsum) {
 				fprintf(f, "month ");
@@ -270,32 +269,25 @@ void sQuotactlListCfg(Service *s, FILE *f){
 				if (q->m.in) fprintf(f, "%s in ", bytesQ2T(q->m.in, buf));
 				if (q->m.out) fprintf(f, "%s out ", bytesQ2T(q->m.out, buf));
 				if (q->m.sum) fprintf(f, "%s sum ", bytesQ2T(q->m.sum, buf));
-				}
+			}
 		
-				if (q->t.in || q->t.out || q->t.softin || q->t.softout || q->t.sum || q->t.softsum) {
-				fprintf(f, "total ");
-				if (q->t.softin) fprintf(f, "%s soft-in ", bytesQ2T(q->t.softin, buf));
-				if (q->t.softout) fprintf(f, "%s soft-out ", bytesQ2T(q->t.softout, buf));
-				if (q->t.softsum) fprintf(f, "%s soft-sum ", bytesQ2T(q->t.softsum, buf));
-				if (q->t.in) fprintf(f, "%s in ", bytesQ2T(q->t.in, buf));
-				if (q->t.out) fprintf(f, "%s out ", bytesQ2T(q->t.out, buf));
-				if (q->t.sum) fprintf(f, "%s sum ", bytesQ2T(q->t.sum, buf));
-				}
 			fprintf(f, "\n");
 			
 			if (q->root) {
 				fprintf(f, "bind quota %s to ", q->name);
-				for (qUlist *ql=q->root;ql!=NULL;ql=ql->next) 
-						if (ql->u) fprintf(f, "%06X ", ql->u->id);
+				qUlist *p=NULL;
+				for (qUlist *ql=q->root;ql!=NULL;ql=ql->next) {
+					if (ql->u) fprintf(f, "%06X ", ql->u->id);
 					else {
-						if(q->root == ql) q->root=ql->next; else ql->prev->next=ql->next;
-                                                 if(ql->next) ql->next->prev=ql->prev;
+						if(q->root == ql) q->root=ql->next; 
+						else p->next=ql->next;
 					}
-				fprintf(f, "\n");
+					p=ql;
 				}
+				fprintf(f, "\n");
 			}
-
 		}
+	}
 
 	if (cfg->user_soft_reach || cfg->soft_reach) {
 		fprintf(f, "alert soft-reach ");
@@ -324,7 +316,7 @@ void sQuotactlListCfg(Service *s, FILE *f){
 	}
 
 //////////////////////////////////////////////////////////////////////////
-void cShowQuotactl(Connection *conn, char *param[32]){
+void cShowQuotactl(Connection *conn, char *param[]){
 	char buf[32], buf2[32];
 	Qdef *q;
 	NetUnit *u;
@@ -332,12 +324,12 @@ void cShowQuotactl(Connection *conn, char *param[32]){
  	Service *s=NULL;
 	char *r_qname=NULL; oid r_id=0;
 
-	for(unsigned i=2;param[i]!=empty;) {
+	for(u_char i=2;param[i]!=empty;) {
 		if (!strcasecmp("name", param[i])) {
 		 	if (param[i+1]!=empty) r_qname=param[i+1];
 			i+=2;
 		} 
-		else if (!strcasecmp("oid", param[i])) {
+		else if (!strcasecmp("unit", param[i])) {
 		 	if (param[i+1]!=empty) {
 				if((u=Units.getUnit(param[i+1]))) r_id=u->id;
 				else r_id=strtol(param[i+1], NULL, 16);
@@ -346,7 +338,7 @@ void cShowQuotactl(Connection *conn, char *param[32]){
 		} else i++;
 	}
         
-	while((s=Services.getServiceByName("quotactl",s))) {
+	while((s=Services.getServiceNextByType(SERVICE_QUOTACTL,s))) {
 		Quotactl_cfg *cfg=(Quotactl_cfg*)s->cfg;
 		if (!cfg->rwlock) return;
 		fprintf(conn->stream_w,"service quotactl %u\n",s->instance);
@@ -358,19 +350,24 @@ void cShowQuotactl(Connection *conn, char *param[32]){
 		if (q->name) if ((r_qname && !strcasecmp(q->name, r_qname)) || r_qname==NULL)
 			{
 			fprintf(conn->stream_w, "QUOTA: %s\tpolicy %s\t", q->name, (q->policy && q->policy->name)?q->policy->name:"<>");
-
-			ShowSysPolicy(buf, q->sys_policy,q->sys_policy_perm, q->sys_policy_name);    
-			fprintf(conn->stream_w, "set %s\n", buf);
-
+			fprintf(conn->stream_w, "set ");
+			PrintSysPolicy(conn->stream_w, q->sys_policy,q->sys_policy_perm);    
+			fprintf(conn->stream_w, "\n");
+			
+			qUlist *p=NULL;
 			for (qUlist *ql=q->root;ql!=NULL;ql=ql->next) { 
 				if (ql->u) {
 					if (r_id==0 || (r_id!=0 && r_id==ql->u->id) ) {
 						u=ql->u;
+						if(!u->ap) goto NEXT;
 
-						ShowSysPolicy(buf, u->sys_policy, u->sys_policy_perm, u->sys_policy_name);    
-						pd=u->ap.Get(q->policy);
-						fprintf(conn->stream_w, " UNIT %06X\t(%s)\tSYST: %s\tACCT is %spresent\n", u->id, u->name?u->name:"<>", buf, pd?"":"NOT ");
-						if (!pd) continue;
+						pd=u->ap->Get(q->policy);
+						fprintf(conn->stream_w, " UNIT %06X\t(%s)\tSYST: ",u->id, u->name?u->name:"<>");
+						PrintSysPolicy(conn->stream_w, u->sys_policy, u->sys_policy_perm);
+						fprintf(conn->stream_w, "\tACCT is %spresent\n", pd?"":"NOT ");
+						if (!pd) goto NEXT;
+						
+						pthread_rwlock_rdlock(u->ap->rwlock);					
 
 						if (q->h.softin)  	fprintf(conn->stream_w, "  HOUR   in: %s, softquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->h.in, buf), bytesQ2T(q->h.softin, buf2), 100.0*(double)pd->h.in/q->h.softin, pd->h.in>=q->h.softin?'-':'+');
 						if (q->h.in)  		fprintf(conn->stream_w, "  HOUR   in: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->h.in, buf), bytesQ2T(q->h.in, buf2), 100.0*(double)pd->h.in/q->h.in, pd->h.in>=q->h.in?'-':'+');
@@ -399,19 +396,15 @@ void cShowQuotactl(Connection *conn, char *param[32]){
 						if (q->m.out) 		fprintf(conn->stream_w, "  MONTH out: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->m.out, buf), bytesQ2T(q->m.out, buf2), 100.0*(double)pd->m.out/q->m.out, pd->m.out>=q->m.out?'-':'+');
 						if (q->m.softsum) 		fprintf(conn->stream_w, "  MONTH sum: %s, softquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->m.in+pd->m.out, buf), bytesQ2T(q->m.softsum, buf2), 100.0*(double)(pd->m.in+pd->m.out)/q->m.softsum, (pd->m.in+pd->m.out)>=q->m.softsum?'-':'+');
 						if (q->m.sum) 		fprintf(conn->stream_w, "  MONTH sum: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->m.in+pd->m.out, buf), bytesQ2T(q->m.sum, buf2), 100.0*(double)(pd->m.in+pd->m.out)/q->m.sum, (pd->m.in+pd->m.out)>=q->m.sum?'-':'+');
-
-						if (q->t.softin)  		fprintf(conn->stream_w, "  TOTAL  in: %s, softquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.in, buf), bytesQ2T(q->t.softin, buf2), 100.0*(double)pd->t.in/q->t.softin, pd->t.in>=q->t.softin?'-':'+');
-						if (q->t.in)  		fprintf(conn->stream_w, "  TOTAL  in: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.in, buf), bytesQ2T(q->t.in, buf2), 100.0*(double)pd->t.in/q->t.in, pd->t.in>=q->t.in?'-':'+');
-						if (q->t.softout) 		fprintf(conn->stream_w, "  TOTAL out: %s, softquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.out, buf), bytesQ2T(q->t.softout, buf2), 100.0*(double)pd->t.out/q->t.softout, pd->t.out>=q->t.softout?'-':'+');
-						if (q->t.out) 		fprintf(conn->stream_w, "  TOTAL out: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.out, buf), bytesQ2T(q->t.out, buf2), 100.0*(double)pd->t.out/q->t.out, pd->t.out>=q->t.out?'-':'+');
-						if (q->t.softsum) 		fprintf(conn->stream_w, "  TOTAL sum: %s, softquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.in+pd->t.out, buf), bytesQ2T(q->t.softsum, buf2), 100.0*(double)(pd->t.in+pd->t.out)/q->t.softsum, (pd->t.in+pd->t.out)>=q->t.softsum?'-':'+');
-						if (q->t.sum) 		fprintf(conn->stream_w, "  TOTAL sum: %s, hardquota %s ratio %.2f%% -> [%c]\n", bytesQ2T(pd->t.in+pd->t.out, buf), bytesQ2T(q->t.sum, buf2), 100.0*(double)(pd->t.in+pd->t.out)/q->t.sum, (pd->t.in+pd->t.out)>=q->t.sum?'-':'+');
-						}
-
+						
+						pthread_rwlock_unlock(u->ap->rwlock);
+					}
 				} else {
-                                	if(q->root == ql) q->root=ql->next; else ql->prev->next=ql->next;
-                                        if(ql->next) ql->next->prev=ql->prev; 
-                                       }
+                                	if(q->root == ql) q->root=ql->next; 
+					else p->next=ql->next;
+				}
+NEXT:
+				p=ql;
 			}
 			fprintf(conn->stream_w, "\n");
 		}
@@ -427,30 +420,36 @@ void *sQuotactl(void *ss){
 	Qdef *q;
 	NetUnit *u;
 	policy_data *pd;
-	unsigned q_viol, q_soft;
+	u_short q_viol, q_soft;
+	time_t now;
 
 	aLog(D_INFO, "service quotactl:%u thread started (%p)\n", s->instance, s->cfg);
 	pthread_cleanup_push(sQuotactlCancel, s);
-	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+	SET_CANCEL();
 	s->t_id=pthread_self();
 
 	// now we should sleep before starting service
-	s->Sleep();
+	if(!(s->flags&SERVICE_RUN)) s->Sleep();
 
-	aLog(D_DEBUG, "service quotactl:%u checking every %d seconds\n", s->instance, cfg->delay/1000);
+	aLog(D_DEBUG, "service quotactl:%u checking every %d seconds\n", s->instance, cfg->delay);
 	while (1) {
+		now=time(NULL);
 		pthread_rwlock_wrlock(cfg->rwlock);
 	
 		for (q=cfg->root; q!=NULL; q=q->next)
 		if (q->name) {
+			qUlist *p=NULL;
 			for (qUlist *ql=q->root;ql!=NULL;ql=ql->next) { 
 				if (ql->u) {
 					q_viol=q_soft=0;
 					u=ql->u;
-
-					pd=u->ap.Get(q->policy);
-					if (!pd) continue;
-
+					if (!u->ap) goto NEXT;
+					
+					pd=u->ap->Get(q->policy);
+					if (!pd) goto NEXT;
+					
+					pthread_rwlock_rdlock(u->ap->rwlock);
+					
 					if (q->h.in) if (pd->h.in>=q->h.in) q_viol=1;
 					if (q->h.out) if (pd->h.out>=q->h.out) q_viol=1;
 					if (q->h.sum) if ((pd->h.in+pd->h.out)>=q->h.sum) q_viol=1;
@@ -479,60 +478,56 @@ void *sQuotactl(void *ss){
 					if (q->m.softout) if (pd->m.out>=q->m.softout) q_soft=1;
 					if (q->m.softsum) if ((pd->m.in+pd->m.out)>=q->m.softsum) q_soft=1;
 					
-					if (q->t.in) if (pd->t.in>=q->t.in) q_viol=1;
-					if (q->t.out) if (pd->t.out>=q->t.out) q_viol=1;
-					if (q->t.sum) if ((pd->t.in+pd->t.out)>=q->t.sum) q_viol=1;
-					if (q->t.softin) if (pd->t.in>=q->t.softin) q_soft=1;
-					if (q->t.softout) if (pd->t.out>=q->t.softout) q_soft=1;
-					if (q->t.softsum) if ((pd->t.in+pd->t.out)>=q->t.softsum) q_soft=1;
+					pthread_rwlock_unlock(u->ap->rwlock);				
 
-					if (!q_viol && u->sys_policy==q->sys_policy) {
-						u->sys_policy=SP_NONE;
-						u->sys_policy_name=SysPolicy_name[SP_NONE];
+					if (!q_viol && (u->sys_policy&q->sys_policy)) {
+						u->SetSysPolicy(q->sys_policy, REMOVE, now);
 						u->sys_policy_perm=0;
 						aLog(D_WARN, "unit %06X (%s) quota %s reset back\n", u->id, u->name?u->name:"<>", q->name);
 						sQuotaSendAlert(s, u, q, 0);
 						ql->q_soft=ql->q_viol=0;
 						cAccessScriptCall(PASS, u, "QUOTA RESET");
-						}
-					if (q_viol && u->sys_policy!=q->sys_policy) {
-						u->sys_policy=q->sys_policy;
-						u->sys_policy_name=q->sys_policy_name;
+					}
+					if (q_viol && !(u->sys_policy&q->sys_policy)) {
+						u->SetSysPolicy(q->sys_policy, ADD, now);
 						u->sys_policy_perm=q->sys_policy_perm;
 						aLog(D_WARN, "unit %06X (%s) violated quota %s\n", u->id, u->name?u->name:"<>", q->name);
 						sQuotaSendAlert(s, u, q, 1);
 						ql->q_viol=q_viol;
 						cAccessScriptCall(DROP, u, "QUOTA VIOLATE");
-						}
-					if (q_soft && u->sys_policy!=q->sys_policy && !ql->q_soft) {
+					}
+					if (q_soft && !(u->sys_policy&q->sys_policy) && !ql->q_soft) {
 						aLog(D_WARN, "unit %06X (%s) reached soft quota %s\n", u->id, u->name?u->name:"<>", q->name);
 						sQuotaSendAlert(s, u, q, 2);
 						ql->q_soft=q_soft;
-						}
+					}
 					if (!q_soft && ql->q_soft) {
 						aLog(D_WARN, "unit %06X (%s) soft quota %s reset back\n", u->id, u->name?u->name:"<>", q->name);
 						ql->q_soft=0;
-						}
-
+					}
 		   		} else {
-                                   	if(q->root == ql) q->root=ql->next; else ql->prev->next=ql->next;
-                                   	if(ql->next) ql->next->prev=ql->prev;
-                                       }
+                                   	if(q->root == ql) q->root=ql->next; 
+					else p->next=ql->next;
 				}
+NEXT:				
+				p=ql;
 			}
-		pthread_rwlock_unlock(cfg->rwlock);
-		s->Sleep(cfg->delay); // quotas will be checked every cfg->delay/1000 seconds
 		}
+		pthread_rwlock_unlock(cfg->rwlock);
+		
+		s->Sleep(cfg->delay); // quotas will be checked every cfg->delay seconds
+	}
 
 	pthread_cleanup_pop(0);
 	return NULL;
-	}
+}
 
 //////////////////////////////////////////////////////////////////////////
 void sQuotactlCancel(void *v){
 	Service *s = (Service*)v;
 	Quotactl_cfg *cfg=(Quotactl_cfg*)s->cfg;
-	
+	time_t now=time(NULL);
+
 	aLog(D_INFO, "cancelling service quotactl:%u\n", s->instance);
 
 	pthread_rwlock_wrlock(cfg->rwlock);
@@ -541,10 +536,9 @@ void sQuotactlCancel(void *v){
 	for (Qdef *q=cfg->root; q!=NULL;) {
 		for(qUlist *ql=q->root;ql!=NULL;) {
                 	NetUnit *u=ql->u;
-                        if (u && u->sys_policy==q->sys_policy) {
-                        	u->sys_policy=SP_NONE;
-                                u->sys_policy_name=SysPolicy_name[SP_NONE];
-                                u->sys_policy_perm=0;
+                        if (u && (u->sys_policy&q->sys_policy)) {
+				u->SetSysPolicy(q->sys_policy, REMOVE, now);	
+                                if(q->sys_policy_perm) u->sys_policy_perm=0;
 			}
 			qUlist *ptr=ql;
 			ptr=ql;
@@ -563,12 +557,11 @@ void sQuotactlCancel(void *v){
 	aFree(s->cfg);
 }
 //////////////////////////////////////////////////////////////////////////
-void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, int dir){	 // dir=1:violates; =0:back; =2:soft_quota_reached
+void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, u_char dir){	 // dir=1:violates; =0:back; =2:soft_quota_reached
 	if(!u->email) return; //nowhere to send
 	Quotactl_cfg *cfg=(Quotactl_cfg*)s->cfg;
 	Service *ss=NULL;
-        if(!(ss=Services.getServiceByName("alerter",ss))) return;
-
+        if(!(ss=Services.getServiceNextByType(SERVICE_ALERTER,ss))) return;
 	ServiceAlerter_cfg *alerter=(ServiceAlerter_cfg*)ss->cfg;
 
 	if (!alerter->queue) return; // alerter service is no running
@@ -576,11 +569,9 @@ void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, int dir){	 // dir=1:violate
 	Message *msg;
 	alert *al;
 
-	msg = new Message();
-	msg->type=ALERT;
-	al=(alert*)aMalloc(sizeof(alert));
+	msg = MsgMgr.New(MSG_ALERT);
 	
-	msg->al=al;
+	al=((Message_Alert*)msg)->al;
 	al->sent=time(NULL);
 	al->expired=al->sent+60*60; // one hour expire
 	al->report_id=0x06101; 
@@ -592,7 +583,7 @@ void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, int dir){	 // dir=1:violate
 	buffer=(char*)aMalloc(255);
 	timeU2T(time(NULL), buffer);
 
-	print_to_string(&message, "This is automatically generated report by %s, version %d.%d(%d)\nTime: %s\n\n", aaa_fw_software_name, aaa_fw_major_version, aaa_fw_minor_version, aaa_fw_build_version, buffer);
+	print_to_string(&message, "This is automatically generated report by %s, version %d.%d.%d (build %d)\nTime: %s\n\n", aaa_fw_software_name, aaa_fw_major_version, aaa_fw_minor_version, aaa_fw_subversion, aaa_fw_build_version, buffer);
 	
 	//we do not need lock here because we locked it in sQuotactl already
 
@@ -636,12 +627,12 @@ void sQuotaSendAlert(Service *s,NetUnit *u, Qdef *q, int dir){	 // dir=1:violate
 	aDebug(DEBUG_ALERT, "alert (quota) %u complete, data is %d bytes\n", al->alert_id, strlen(al->data));
 }
 //////////////////////////////////////////////////////////////////////////
-void sQuotactlGetValue(int *i, char *param[], qstat *q){
+void sQuotactlGetValue(u_char *i, char *param[], qstat *q){
 
-	int j=*i+1;
+	u_short j=*i+1;
 	unsigned long long data;
 	while (1) {
-		bytesT2Q(&data, param[j]);
+		data=bytesT2Q(param[j]);
 
 		if (strcasecmp(param[j+1], "in")==0) q->in=data;
 		else if (strcasecmp(param[j+1], "out")==0) q->out=data;
