@@ -1,13 +1,13 @@
 /*************************************************************************
-***     Authentication, authorization, accounting + firewalling package
-***     (c) 1998-2003 Anton Vinokurov <anton@netams.com>
-***		(c) 2003 NeTAMS Development Team
-***		All rights reserved. See 'Copying' file included in distribution
-***		For latest version and more info, visit this project web page
-***		located at http://www.netams.com
-***		 					
+***	Authentication, authorization, accounting + firewalling package
+***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
+***	(c) 2002-2005 NeTAMS Development Team
+***	All rights reserved. See 'Copying' file included in distribution
+***	For latest version and more info, visit this project web page
+***	located at http://www.netams.com
+***
 *************************************************************************/
-/* $Id: security.h,v 1.4.2.1 2004/05/06 10:35:06 jura Exp $ */
+/* $Id: security.h,v 1.7 2005/01/17 13:13:22 jura Exp $ */
 
 /////////////////////////////////////////////////////////////////////////////////////////
 // Permissions Management ///////////////////////////////////////////////////////////////////
@@ -28,5 +28,5 @@ extern char *permissions_list[256];
 #define UPERM_ALL           255	 
 
 void aPermissionsInit();
-unsigned aCheckPerm(Connection *conn, unsigned perm);
+u_char aCheckPerm(Connection *conn, u_char perm);
 int aAuth(char *login, char *password);
