@@ -1,31 +1,43 @@
 #!/bin/sh
-# $Id: versions.sh,v 1.5.2.1 2004/05/06 12:23:43 jura Exp $ 
+# $Id: versions.sh,v 1.14 2005/01/07 00:22:50 anton Exp $ 
 
-build_hostname=`hostname -s`
+build_hostname=`uname -n`
 current_v=`cat versions_build.dat`
 next_v=$current_v
 next_v_l=0;
 
-if [ $build_hostname != "srv" -a $build_hostname != "notebook" -a $build_hostname != "test" ]; then
+os=`uname`
+if [ $os = "SunOS" ] 
+then
+	sh=ksh
+else 
+	sh=sh
+fi
+
+if [ ! -d ../private ]; then
 	if [ -r versions_build_local.dat ]; then
 		current_v_l=`cat versions_build_local.dat`
 	else
 		current_v_l=0;
 	fi
-	next_v_l=$((1+$current_v_l))
+		
+	next_v_l=`$sh -c "echo $((1+$current_v_l))"`
 	echo $next_v_l > versions_build_local.dat
 
 else
-	next_v=$((1+$current_v))
+	next_v=`$sh -c "echo $((1+$current_v))"`
 	next_v_l=0;
 	rm -rf versions_build_local.dat 
 	echo $next_v > versions_build.dat
 fi
 	
-
-
 build_date=`date`
-build_person=`whoami`@`hostname -s`
+if [ $os = "SunOS" ]
+then
+	build_person=`id |cut -f 1 -d ' '|cut -f 2 -d '('|sed s/\)//`@`uname -n`
+else 
+	build_person=`whoami`@`uname -n`
+fi
 
 echo "const unsigned int aaa_fw_build_version=$next_v;" > versions_build.c
 echo "const unsigned int aaa_fw_build_version_local=$next_v_l;" >> versions_build.c
@@ -43,4 +55,4 @@ if [ "$2" != "" ]; then
 fi
 echo "linking for version $next_v.$next_v_l"
 rm -rf versions_build.o
-cc -c versions_build.c
+gcc -c versions_build.c
